import { Request, Response } from "express";
import MemberService from "../../../service/configuration/memberService";
import MemberFactory from "../../../factory/admin/configuration/member";
import {
  CreateMemberCommand,
  DeleteMemberCommand,
  UpdateMemberCommand,
} from "../../../command/configuration/member/memberCommand";
import MemberCommandHandler from "../../../command/configuration/member/memberCommandHandler";

/**
 * @description get all members
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function getAllMembers(req: Request, res: Response): Promise<any> {
  let offset = parseInt((req.query.offset as string) ?? "0");
  let count = parseInt((req.query.count as string) ?? "25");
  let search = (req.query.search as string) ?? "";
  let noLimit = req.query.noLimit === "true";
  let ids = ((req.query.ids ?? "") as string).split(",").filter((i) => i);

  let [members, total] = await MemberService.getAll({ offset, count, search, noLimit, ids });

  res.json({
    members: MemberFactory.mapToBase(members),
    total: total,
    offset: offset,
    count: count,
  });
}

/**
 * @description get members by Ids
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function getMembersByIds(req: Request, res: Response): Promise<any> {
  let ids = req.body.ids as Array<string>;

  let [members, total] = await MemberService.getAll({ noLimit: true, ids });

  res.json({
    members: MemberFactory.mapToBase(members),
    total: total,
    offset: 0,
    count: total,
  });
}

/**
 * @description get member by id
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function getMemberById(req: Request, res: Response): Promise<any> {
  const memberId = req.params.id;
  let member = await MemberService.getById(memberId);

  res.json(MemberFactory.mapToSingle(member));
}

/**
 * @description create member
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function createMember(req: Request, res: Response): Promise<any> {
  const salutationId = parseInt(req.body.salutationId);
  const firstname = req.body.firstname;
  const lastname = req.body.lastname;
  const nameaffix = req.body.nameaffix;
  const birthdate = req.body.birthdate;
  const internalId = req.body.internalId || null;

  let createMember: CreateMemberCommand = {
    salutationId,
    firstname,
    lastname,
    nameaffix,
    birthdate,
    internalId,
  };
  let memberId = await MemberCommandHandler.create(createMember);

  res.status(200).send(memberId);
}

/**
 * @description update member by id
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function updateMemberById(req: Request, res: Response): Promise<any> {
  const memberId = req.params.id;
  const salutationId = parseInt(req.body.salutationId);
  const firstname = req.body.firstname;
  const lastname = req.body.lastname;
  const nameaffix = req.body.nameaffix;
  const birthdate = req.body.birthdate;
  const internalId = req.body.internalId || null;

  let updateMember: UpdateMemberCommand = {
    id: memberId,
    salutationId,
    firstname,
    lastname,
    nameaffix,
    birthdate,
    internalId,
  };
  await MemberCommandHandler.update(updateMember);

  res.sendStatus(204);
}

/**
 * @description delete member by id
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function deleteMemberById(req: Request, res: Response): Promise<any> {
  const memberId = req.params.id;

  let deleteMember: DeleteMemberCommand = {
    id: memberId,
  };
  await MemberCommandHandler.delete(deleteMember);

  res.sendStatus(204);
}
