import { dataSource } from "../../data-source";
import { member } from "../../entity/configuration/member";
import DatabaseActionException from "../../exceptions/databaseActionException";

export default abstract class MemberService {
  /**
   * @description get all members
   * @returns {Promise<[Array<member>, number]>}
   */
  static async getAll({
    offset = 0,
    count = 25,
    search = "",
    noLimit = false,
    ids = [],
  }: {
    offset?: number;
    count?: number;
    search?: string;
    noLimit?: boolean;
    ids?: Array<string>;
  }): Promise<[Array<member>, number]> {
    let query = dataSource.getRepository(member).createQueryBuilder("member");

    if (search != "") {
      search.split(" ").forEach((term, index) => {
        const searchQuery = `%${term}%`;
        const dynamic = "searchQuery" + Math.random().toString(36).substring(2);
        if (index == 0) {
          query = query.where(`member.firstname LIKE :${dynamic} OR member.lastname LIKE :${dynamic}`, {
            [dynamic]: searchQuery,
          });
        } else {
          query = query.orWhere(`member.firstname LIKE :${dynamic} OR member.lastname LIKE :${dynamic}`, {
            [dynamic]: searchQuery,
          });
        }
      });
    }

    if (ids.length != 0) {
      query = query.where("member.id IN (:...ids)", { ids: ids });
    }

    if (!noLimit) {
      query = query.offset(offset).limit(count);
    }

    return await query
      .orderBy("member.lastname")
      .addOrderBy("member.firstname")
      .addOrderBy("member.nameaffix")
      .getManyAndCount()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new DatabaseActionException("SELECT", "member", err);
      });
  }

  /**
   * @description get member by id
   * @param {string} id
   * @returns {Promise<member>}
   */
  static async getById(id: string): Promise<member> {
    return dataSource
      .getRepository(member)
      .createQueryBuilder("member")
      .where("member.id = :id", { id: id })
      .getOneOrFail()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new DatabaseActionException("SELECT", "member", err);
      });
  }
}
